$ifndef __ALCD_INC_
$define __ALCD_INC_
'
' Alphanumeric LCD interface with defaults for the FireWing R2 LCD shield
'   
' LCD interface...
'
$ifndef ALCD_Interface
$define ALCD_Interface 4                ' FireWing R2 LCD shield uses a 4-bit interface
$endif
'
' Command timeout...
'
$ifndef ALCD_CommandUs
$define ALCD_CommandUs 2000             ' Command delay in micro seconds
$endif
'
' data timeout...
'
$ifndef ALCD_DataUs
$define ALCD_DataUs 50                   ' Data delays in micro seconds
$endif
'
' Module initialisation delay...
'
$ifndef ALCD_InitDelay
$define ALCD_InitDelay 100              ' Module startup delay in milliseconds
$endif
'
' RS pin...
'
$ifndef ALCD_RS_Pin
    $define ALCD_RS_Pin PORTB.10        ' FireWing R2 pin assignement
$endif
'
' EN pin...
'
$ifndef ALCD_EN_Pin
    $define ALCD_EN_Pin PORTA.2         ' FireWing R2 pin assignement
$endif
'
' By default, the module assumes RW is not used and tied to ground
' If the user specifies a RW pin, then uncomment the $define below and assign a pin to it
'
'   $define ALCD_RW_Pin PORTX.X
'
' Standard DDRAM line offsets
'
$ifdef LCD_DOGM
    $if LCD_DOGM = DOGM_163
        Symbol cALCD_LINE_2 = $10
        Symbol cALCD_LINE_3 = $20
    $else
        Symbol cALCD_LINE_2 = $40
        Symbol cALCD_LINE_3 = $14
        Symbol cALCD_LINE_4 = $54
    $endif
$else
    Symbol cALCD_LINE_2 = $40
    Symbol cALCD_LINE_3 = $14
    Symbol cALCD_LINE_4 = $54
$endif

$if ALCD_Interface = 8              ' FireWing R2 Pin assignments for 8 bit mode...
    Symbol _ALCD_tD0 = PORTB.6
    Symbol _ALCD_tD1 = PORTB.7
    Symbol _ALCD_tD2 = PORTB.8
    Symbol _ALCD_tD3 = PORTB.9
    Symbol _ALCD_tD4 = PORTA.3
    Symbol _ALCD_tD5 = PORTB.4
    Symbol _ALCD_tD6 = PORTB.5
    Symbol _ALCD_tD7 = PORTB.12
$else                               ' FireWing R2 Pin assignments for 4 bit mode...
    Symbol _ALCD_tD0 = PORTA.3
    Symbol _ALCD_tD1 = PORTB.4
    Symbol _ALCD_tD2 = PORTB.5
    Symbol _ALCD_tD3 = PORTB.12
$endif

    Symbol cALCD_cmdCGRAM     = %01000000
    Symbol cALCD_cmdDDRAM     = %10000000
    Symbol cALCD_cmdClear     = %00000001
    Symbol cALCD_cmdHome      = %00000010
    Symbol cALCD_cmdCursorOff = %00001100
    Symbol cALCD_cmdCursorOn  = %00001110
    Symbol cALCD_cmdBlinkOn   = %00001101
    Symbol cALCD_cmdBlinkOff  = %00001100
    Symbol cALCD_cmdMoveLeft  = %00010000
    Symbol cALCD_cmdMoveRight = %00010100

' bring pin options into the module...
    Symbol _ALCD_tDataPin0 = _ALCD_tD0
    Symbol _ALCD_tDataPin1 = _ALCD_tD1
    Symbol _ALCD_tDataPin2 = _ALCD_tD2
    Symbol _ALCD_tDataPin3 = _ALCD_tD3

$if ALCD_Interface = 8
    Symbol _ALCD_tDataPin4 = _ALCD_tD4
    Symbol _ALCD_tDataPin5 = _ALCD_tD5
    Symbol _ALCD_tDataPin6 = _ALCD_tD6
    Symbol _ALCD_tDataPin7 = _ALCD_tD7
$endif

    Dim ALCD_bPosX As Byte
    Dim ALCD_bPosY As Byte              ' Local x, y coordinates

'------------------------------------------------------------------------------------------------------
' Strobe read enable - assumes ALCD_EN_Pin is in zero state.
' Rather than halt the processor after ALCD_EN_Pin = 0, the procedure returns immediately
' Input     : None
' Output    : None
' Notes     : None
'
$define ALCD_StrobeEN() '
    Set ALCD_EN_Pin     '
    DelayUS 1           '
    Clear ALCD_EN_Pin

'------------------------------------------------------------------------------------------------------
' Make data pins output
' Input     : None
' Output    : None
' Notes     : None
'
Proc ALCD_MakeOutput()
    Output _ALCD_tDataPin0
    Output _ALCD_tDataPin1
    Output _ALCD_tDataPin2
    Output _ALCD_tDataPin3

$if ALCD_Interface = 8
    Output _ALCD_tDataPin4
    Output _ALCD_tDataPin5
    Output _ALCD_tDataPin6
    Output _ALCD_tDataPin7
$endif
EndProc

'------------------------------------------------------------------------------------------------------
' Make data pins input
' Input     : None
' Output    : None
' Notes     : None
'
$ifdef ALCD_RW_Pin                      ' Is the RW line assigned?
Proc ALCD_MakeInput()
   Input _ALCD_tDataPin0
   Input _ALCD_tDataPin1
   Input _ALCD_tDataPin2
   Input _ALCD_tDataPin3

$if ALCD_Interface = 8
   Input _ALCD_tDataPin4
   Input _ALCD_tDataPin5
   Input _ALCD_tDataPin6
   Input _ALCD_tDataPin7
$endif 
EndProc

'------------------------------------------------------------------------------------------------------
' Read _data port value. For 4 bit interface, value is returned in LSNibble
' Input     : None
' Output    : None
' Notes     : None
'
Proc ALCD_GetData(), Byte
    Set ALCD_EN_Pin
    DelayUS 1

    Result = 0
    Result.0 = _ALCD_tDataPin0
    Result.1 = _ALCD_tDataPin1
    Result.2 = _ALCD_tDataPin2
    Result.3 = _ALCD_tDataPin3

$if ALCD_Interface = 8
    Result.4 = _ALCD_tDataPin4
    Result.5 = _ALCD_tDataPin5
    Result.6 = _ALCD_tDataPin6
    Result.7 = _ALCD_tDataPin7
$endif
    Clear ALCD_EN_Pin
EndProc

'------------------------------------------------------------------------------------------------------
' Read a data byte from the device
' Input     : None
' Output    : Returns the byte read from the LCD
' Notes     : None
'
Proc ALCD_ReadByte(), Byte
    ALCD_MakeInput()
    Clear ALCD_RS_Pin
    Set ALCD_RW_Pin
$if ALCD_Interface = 8
    Result = ALCD_GetData()
$else
    Result = ALCD_GetData() << 4                 ' High nibble
    Result = Result | (ALCD_GetData() & $0F)     ' Low nibble
$endif
   Clear ALCD_RW_Pin
   ALCD_MakeOutput()
EndProc

'------------------------------------------------------------------------------------------------------
' Wait for device's busy flag to clear
' Input     : None
' Output    : None
' Notes     : None
'
Proc ALCD_WaitFor()
    Repeat
    Until (ALCD_ReadByte() & $80) = 0
EndProc
$endif ' ALCD_RW_Pin

'------------------------------------------------------------------------------------------------------
' Sets Data port value
' Input     : pData is the value written to the LCD's data port
' Output    : None
' Notes     : None
'
Proc ALCD_SetData(pData As Byte)
    _ALCD_tDataPin0 = pData.0
    _ALCD_tDataPin1 = pData.1
    _ALCD_tDataPin2 = pData.2
    _ALCD_tDataPin3 = pData.3

$if ALCD_Interface = 8
    _ALCD_tDataPin4 = pData.4
    _ALCD_tDataPin5 = pData.5
    _ALCD_tDataPin6 = pData.6
    _ALCD_tDataPin7 = pData.7
$endif
    ALCD_StrobeEN()
EndProc

'------------------------------------------------------------------------------------------------------
' Set DDRAM address
' Input     : pAddress holds the DDRAM address
' Output    : None
' Notes     : None
'
Proc ALCD_SetDDRAM(pAddress As Byte)
$ifdef ALCD_RW_Pin
    ALCD_WaitFor()
$endif

    Clear ALCD_RS_Pin
$if ALCD_Interface = 8
    ALCD_SetData(pAddress | %10000000)
$else
    ALCD_SetData((pAddress | %10000000) >> 4)
    ALCD_SetData(pAddress)
$endif

$ifndef ALCD_RW_Pin
    DelayUS ALCD_DataUs
$endif
EndProc

'------------------------------------------------------------------------------------------------------
' Write byte command to LCD
' Input     : pCmd holds the commadn to write to the LCD
' Output    : None
' Notes     : None
'
Proc ALCD_WriteCommand(pCmd As Byte)
$ifdef ALCD_RW_Pin
    ALCD_WaitFor()
$endif

    Clear ALCD_RS_Pin
$if ALCD_Interface = 8
    ALCD_SetData(pCmd)
$else
    ALCD_SetData(pCmd >> 4)
    ALCD_SetData(pCmd)
$endif

$ifndef ALCD_RW_Pin
    DelayUS ALCD_CommandUs
$endif
EndProc

'------------------------------------------------------------------------------------------------------
' Write a single byte to the LCD
' Input     : pData holds the value to write to the LCD
' Output    : None
' Notes     : None
'
Proc ALCD_WriteByte(pData As Byte)
$ifdef ALCD_RW_Pin
   ALCD_WaitFor()
$endif

   Set ALCD_RS_Pin
$if ALCD_Interface = 8
   ALCD_SetData(pData)
$else
   ALCD_SetData(pData >> 4)
   ALCD_SetData(pData)
$endif

$ifndef ALCD_RW_Pin
   DelayUS ALCD_DataUs
$endif
EndProc
'------------------------------------------------------------------------------------------------------
' Set the cursor to line and column
' Input     : pLine holds the line (0 to 3) of interest
'           : pColumn holds the column of interest
' Output    : None
' Notes     : None
'
Proc ALCD_SetCursor(pLine As Byte, pColumn As Byte)
   Dec pColumn
   Select pLine
      Case 1
        ALCD_SetDDRAM(pColumn)
      Case 2
        ALCD_SetDDRAM(cALCD_LINE_2 + pColumn)
      Case 3
        ALCD_SetDDRAM(cALCD_LINE_3 + pColumn)
      Case 4
        ALCD_SetDDRAM(cALCD_LINE_4 + pColumn)
   EndSelect
EndProc

'------------------------------------------------------------------------------------------------------
' Moves cursor to module private x, y location
' Input     : ALCD_bPosY holds the line (0 to 3) of interest
'           : ALCD_bPosX holds the column of interest
' Output    : None
' Notes     : None
'
$define ALCD_MoveTo() ALCD_SetCursor(ALCD_bPosY, ALCD_bPosX)

'------------------------------------------------------------------------------------------------------
' Set cursor X position
' Input     : pX holds the X position of interest
' Output    : None
' Notes     : None
'
$define ALCD_SetLocationX(pX) ALCD_bPosX = pX

'------------------------------------------------------------------------------------------------------
' Set cursor Y position
' Input     : pY holds the Y position of interest
' Output    : None
' Notes     : None
'
$define ALCD_SetLocationY(pY) ALCD_bPosY = pY

'------------------------------------------------------------------------------------------------------
' Clear the LCD screen
' Input     : None
' Output    : None
' Notes     : None
'
Proc ALCD_Clear()
   ALCD_WriteCommand(cALCD_cmdClear)
   DelayMS 30
EndProc

'------------------------------------------------------------------------------------------------------
' Performs a initialisation sequence
' Input     : pValue holds the value to write to the LCD
' Output    : None
' Notes     : None
'
Proc ALCD_DoInitSequence(pValue As Byte)
    ALCD_SetData(pValue)
    DelayMS 5
    ALCD_SetData(pValue)
    DelayMS 5
    ALCD_SetData(pValue)
    DelayUS 150
EndProc

'------------------------------------------------------------------------------------------------------
' Initialise the LCD
' Input     : None
' Output    : None
' Notes     : None
'
Proc ALCD_Init()
    DelayMS ALCD_InitDelay

' read write...
$ifdef ALCD_RW_Pin
    Low ALCD_RW_Pin
$endif

' set ALCD_RS_Pin and EN to low output...
    Low ALCD_RS_Pin
    Low ALCD_EN_Pin
    DelayMS 15
    ALCD_MakeOutput()

$if ALCD_Interface = 8                  ' Initialise for 8 bit interface...
    ALCD_DoInitSequence($30)
    ALCD_WriteCommand(%00111000)        ' DL = 1, N = 1, F = 0
$else                                   ' Initialise for 4 bit interface...
    ALCD_DoInitSequence($03)
    ALCD_SetData($02)
    DelayUS 160
    ALCD_WriteCommand(%00101000)        ' DL = 0, N = 1, F = 0
$endif

    ALCD_WriteCommand(%00001000)        ' D = 0, C = 0, B = 0
    ALCD_WriteCommand(%00000001)        ' Clear
    ALCD_WriteCommand(%00000110)        ' ID = 1, S = 0
    ALCD_WriteCommand(%00001100)        ' D = 1, C = 0, B = 0
EndProc

$endif  ' __ALCD_INC_

