$ifndef __DOGM_INC_
$define __DOGM_INC_
' Routines for a DOGm alphamumeric LCD
' Default values are set for the FireWing LCD Plus Shield
'
' The procedures can be used standalone, but the library also replaces the compiler's Cursor, Cls, and Print commands
'
' The compiler will produce the hooks:
' __SYSCOM_ALCD_CURSOR_REQ_ for the Cursor command
' __SYSCOM_ALCD_CLS_REQ_ for the Cls command
' __SYSCOM_ALCD_PRINT_REQ_ for the Print command
'
    #disable Print, LCD_Cls, LCD_Cursor         ' Disable the compiler's Alphanumeric LCD library routines

$define LCD_DOGM DOGM_162                       ' Can be DOGM_161, DOGM_162, or DOGM_163

    Include "ALCD.inc"                          ' Load the Alphanumeric LCD procedures into the program

    Symbol cDOGM_MinBacklight = 0
    Symbol cDOGM_MaxBacklight = 1023
    Symbol cDOGM_MinContrast = 0
    Symbol cDOGM_MaxContrast = 32

    Dim DOGM_wBackLight As Word = 0
    Dim DOGM_bContrast As Byte = 7
'
' Create some defines from the alphanumeric library
'
$define DOGM_WriteCommand(pCmd) ALCD_WriteCommand(pCmd)
$define DOGM_WriteByte(pData) ALCD_WriteByte(pData)
$define DOGM_Cursor(pLine, pColumn) ALCD_SetCursor(pLine, pColumn)
$define DOGM_Cls() ALCD_Clear() 

'------------------------------------------------------------------------------------------------------
' Set the LCD Plus backlight
' Input     : pValue holds the 10-bit duty cycle for the PWM (0 to 1023)
' Output    : None
' Notes     : Assumes a FireWing R2 board operating at 32MHz for the PWM pin
'
Proc DOGM_SetBacklight(pValue As Word)
    If DOGM_wBackLight = 0 Then
        PR2 = $03FF                 ' Set timer period (1023)
        T2CON = $8020               ' Enable timer 2, prescale = 64
        PPS_Output(cOut_Pin_RP9, cOut_Fn_OC1)
        Output PORTB.9
        OC1CON = %0000000000000110  ' Enable OC1
    EndIf
    If pValue <= cDOGM_MaxBacklight Then DOGM_wBackLight = pValue
    OC1RS = pValue
EndProc

'------------------------------------------------------------------------------------------------------
' Get the LCD Plus backlight value
' Input     :
' Output    : None
' Notes     : None
'
Proc DOGM_GetBacklight(), Word
    Result = DOGM_wBackLight
EndProc

'------------------------------------------------------------------------------------------------------
' Set the LCD Plus contrast
' Input     : pValue holds the contrast value (0 to 31)
' Output    : None
' Notes     : None
'
Proc DOGM_SetContrast(pValue As Byte)
    If pValue <= cDOGM_MaxContrast Then DOGM_bContrast = pValue
    DOGM_WriteCommand($70 + DOGM_bContrast)
EndProc

'------------------------------------------------------------------------------------------------------
' Get the LCD Plus contrast value
' Input     : None
' Output    : Returns the contrast value
' Notes     : None
'
Proc DOGM_GetContrast(), Byte
    Result = DOGM_bContrast
EndProc
'------------------------------------------------------------------------------------------------------
#ifSym __SYSCOM_ALCD_CURSOR_REQ_, __SYSCOM_ALCD_CLS_REQ_, __SYSCOM_ALCD_PRINT_REQ_
    GoTo _Dogm_Main
#endIfSym
'------------------------------------------------------------------------------------------------------
' Alphanumeric Cursor command replacement
' Input     : WREG6.Byte0 holds the Y position(Line)
'           : WREG0.Byte0 holds the X position
' Output    : None
' Notes     : None
'
#ifSym __SYSCOM_ALCD_CURSOR_REQ_
__cursor:
    WREG1 = WREG0
    ALCD_SetCursor(WREG6.Byte0, WREG1.Byte0) 
    Return
#endIfSym   ' __SYSCOM_ALCD_CURSOR_REQ_
'------------------------------------------------------------------------------------------------
' Alphanumeric Cls command replacement
' Input     : None
' Output    : None
' Notes     : None
'
#ifSym __SYSCOM_ALCD_CLS_REQ_
_alcd__cls_:
    DOGM_Cls()
    Return
#endIfSym   ' __SYSCOM_GEN_CLS_REQ_
'------------------------------------------------------------------------------------------------
' Alphanumeric print command replacement
' Input     : WREG0.Byte0 holds the character to display
' Output    : WREG0.Byte0 still holds the byte displayed
' Notes     : None
'
__print:
#ifSym __SYSCOM_ALCD_PRINT_REQ_
    Push WREG0                  ' Save WREG0
    DOGM_WriteByte(WREG0.Byte0)
    Pop WREG0                   ' Restore WREG0
    Return
#endIfSym   ' __SYSCOM_ALCD_PRINT_REQ_
'------------------------------------------------------------------------------------------------------
' Initialise the DOGM LCD
' Input     : None
' Output    : None
' Notes     : None
'
_Dogm_Main:
    ALCD_Init()                         ' Initialise the alphanumeric LCD
    DOGM_WriteCommand($29)              ' 4 bit interface
    DOGM_WriteCommand($1C)              ' bias set
    DOGM_WriteCommand($52)              ' power control
    DOGM_WriteCommand($69)              ' follower control

    DOGM_SetContrast(DOGM_bContrast)    ' set contrast to default value...

$endif  ' __DOGM_INC_

